@@ -49,15 +49,12 @@
 
 package com.openexchange.caldav;
 
-import gnu.trove.map.TIntObjectMap;
-import gnu.trove.map.hash.TIntObjectHashMap;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.regex.Pattern;
 import com.openexchange.exception.OXException;
 import com.openexchange.folderstorage.Permission;
 import com.openexchange.folderstorage.UserizedFolder;
@@ -84,10 +81,50 @@ import com.openexchange.webdav.protocol.WebdavProtocolException;
  */
 public class Patches {
 
+    /** The external representation of the prefix used for private comments denoting a new time proposal */
+    private static final String COMMENT_PROPOSAL_PREFIX_EXTERNAL = "\u200B\uD83D\uDDD3\u200B ";
+
+    /** The internal representation of the prefix used for private comments denoting a new time proposal */
+    private static final String COMMENT_PROPOSAL_PREFIX_INTERNAL = "\u200B\u0e4f\u200B ";
+
     private Patches() {
     	// prevent instantiation
     }
 
+    /**
+     * Replaces the representation of the prefix used for private comments denoting a new time proposal with another one.
+     *
+     * @param appointment The appointment to patch
+     * @param The prefix to replace
+     * @param The replacement prefix
+     */
+    private static void patchPrivateComments(Appointment appointment, String prefix, String replacement) {
+        patchPrivateComments(appointment.getParticipants(), prefix, replacement);
+        patchPrivateComments(appointment.getUsers(), prefix, replacement);
+    }
+
+    /**
+     * Replaces the representation of the prefix used for private comments denoting a new time proposal with another one.
+     *
+     * @param participants The participants to patch
+     * @param The prefix to replace
+     * @param The replacement prefix
+     */
+    private static void patchPrivateComments(Participant[] participants, String prefix, String replacement) {
+        if (null == participants || 0 == participants.length) {
+            return;
+        }
+        for (Participant participant : participants) {
+            if (Participant.USER == participant.getType() && UserParticipant.class.isInstance(participant)) {
+                UserParticipant userParticipant = (UserParticipant) participant;
+                String message = userParticipant.getConfirmMessage();
+                if (null != message && message.startsWith(prefix)) {
+                    userParticipant.setConfirmMessage(replacement + message.substring(prefix.length()));
+                }
+            }
+        }
+    }
+
     /**
      * {@link Incoming}
      *
@@ -333,7 +370,7 @@ public class Patches {
          */
         public static void adjustTaskStart(Task originalTask, Task updatedTask) {
             Date startDate = updatedTask.containsStartDate() ? updatedTask.getStartDate() : originalTask.getStartDate();
-            if (null != startDate && updatedTask.containsEndDate() && updatedTask.getEndDate().before(startDate)) {
+            if (null != startDate && updatedTask.containsEndDate() && null != updatedTask.getEndDate() && updatedTask.getEndDate().before(startDate)) {
                 /*
                  * remove currently set start date
                  */
@@ -364,6 +401,15 @@ public class Patches {
             }
         }
 
+        /**
+         * Replaces the external representation of the prefix used for private comments denoting a new time proposal to the internal one.
+         *
+         * @param appointment The appointment to adjust prior saving
+         */
+        public static void adjustProposedTimePrefixes(Appointment appointment) {
+            patchPrivateComments(appointment, COMMENT_PROPOSAL_PREFIX_EXTERNAL, COMMENT_PROPOSAL_PREFIX_INTERNAL);
+        }
+
     }
 
     /**
@@ -375,23 +421,10 @@ public class Patches {
      */
     public static final class Outgoing {
 
-        private static final Pattern EMPTY_RDATE = Pattern.compile("^RDATE:\\r\\n", Pattern.MULTILINE);
-
         private Outgoing() {
         	// prevent instantiation
         }
 
-        /**
-         * Removes empty RDATE components without information in iCal strings
-         * (those are generated by the {@link VTimeZone} onsets)
-         *
-         * @param iCal
-         * @return
-         */
-        public static String removeEmptyRDates(final String iCal) {
-        	return EMPTY_RDATE.matcher(iCal).replaceAll("");
-        }
-
         /**
          * Sets the correct start- and enddate in a recurring appointment.
          *
@@ -460,45 +493,6 @@ public class Patches {
             return changeExceptions;
         }
 
-        /**
-         * If not yet set, sets the organizer's participant status to "accepted".
-         *
-         * @param appointment
-         */
-        public static void setOrganizersParticipantStatus(Appointment appointment) {
-            UserParticipant[] users = appointment.getUsers();
-            if (null != users) {
-                int createdBy = appointment.getCreatedBy();
-                TIntObjectMap<UserParticipant> userMap = new TIntObjectHashMap<UserParticipant>();
-                for (UserParticipant userParticipant : users) {
-                    int identifier = userParticipant.getIdentifier();
-                    if (createdBy == identifier && userParticipant.getConfirm() == CalendarObject.NONE) {
-                        userParticipant.setConfirm(CalendarObject.ACCEPT);
-                    }
-                    userMap.put(identifier, userParticipant);
-                }
-
-                Participant[] participants = appointment.getParticipants();
-                if (null != participants) {
-                    for (Participant participant : participants) {
-                        if (UserParticipant.class.isInstance(participant)) {
-                            UserParticipant userParticipant = (UserParticipant) participant;
-                            int identifier = userParticipant.getIdentifier();
-                            UserParticipant up = userMap.get(identifier);
-                            if (up != null && CalendarObject.NONE != up.getConfirm()) {
-                                // prefer confirmation status from users when set
-                                userParticipant.setConfirm(up.getConfirm());
-                                userParticipant.setConfirmMessage(up.getConfirmMessage());
-                            } else if (createdBy == identifier && CalendarObject.NONE == userParticipant.getConfirm()) {
-                                // assume 'accepted' when no confirmation set
-                                userParticipant.setConfirm(CalendarObject.ACCEPT);
-                            }
-                        }
-                    }
-                }
-            }
-        }
-
         /**
          * Adds all user participants to the participants list and removes all
          * group participants.
@@ -623,5 +617,14 @@ public class Patches {
             }
         }
 
+        /**
+         * Replaces the internal representation of the prefix used for private comments denoting a new time proposal to the external one.
+         *
+         * @param appointment The appointment to adjust prior serialization
+         */
+        public static void adjustProposedTimePrefixes(Appointment appointment) {
+            patchPrivateComments(appointment, COMMENT_PROPOSAL_PREFIX_INTERNAL, COMMENT_PROPOSAL_PREFIX_EXTERNAL);
+        }
+
     }
 }
