@@ -49,18 +49,20 @@
 
 package com.openexchange.carddav.resources;
 
+import static com.openexchange.dav.DAVProtocol.protocolException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.util.regex.Pattern;
 import javax.servlet.http.HttpServletResponse;
 import org.jdom2.Element;
 import com.openexchange.ajax.fileholder.IFileHolder;
 import com.openexchange.carddav.GroupwareCarddavFactory;
 import com.openexchange.carddav.Tools;
+import com.openexchange.carddav.photos.PhotoUtils;
 import com.openexchange.contact.ContactService;
 import com.openexchange.contact.vcard.VCardExport;
 import com.openexchange.contact.vcard.VCardImport;
@@ -68,22 +70,24 @@ import com.openexchange.contact.vcard.VCardParameters;
 import com.openexchange.contact.vcard.VCardService;
 import com.openexchange.contact.vcard.storage.VCardStorageService;
 import com.openexchange.dav.DAVProtocol;
+import com.openexchange.dav.DAVUserAgent;
 import com.openexchange.dav.PreconditionException;
 import com.openexchange.dav.resources.CommonResource;
 import com.openexchange.exception.Category;
 import com.openexchange.exception.OXException;
+import com.openexchange.groupware.contact.helpers.ContactField;
 import com.openexchange.groupware.container.Contact;
 import com.openexchange.groupware.tools.mappings.MappedIncorrectString;
 import com.openexchange.groupware.tools.mappings.MappedTruncation;
 import com.openexchange.java.Charsets;
 import com.openexchange.java.Streams;
-import com.openexchange.java.Strings;
+import com.openexchange.tools.webdav.WebDAVRequestContext;
 import com.openexchange.webdav.protocol.WebdavPath;
 import com.openexchange.webdav.protocol.WebdavProperty;
 import com.openexchange.webdav.protocol.WebdavProtocolException;
 
 /**
- * {@link ContactResource2}
+ * {@link ContactResource}
  *
  * @author <a href="mailto:tobias.friedrich@open-xchange.com">Tobias Friedrich</a>
  */
@@ -116,9 +120,8 @@ public class ContactResource extends CommonResource<Contact> {
      */
     public ContactResource(GroupwareCarddavFactory factory, CardDAVCollection parent, Contact object, WebdavPath url) throws OXException {
         super(parent, object, url);
-        this.object = object;
-        this.parent = parent;
         this.factory = factory;
+        this.parent = parent;
     }
 
     /**
@@ -165,11 +168,11 @@ public class ContactResource extends CommonResource<Contact> {
                 try {
                     return vCardResource.getClosingStream();
                 } catch (OXException e) {
-                    throw protocolException(e);
+                    throw protocolException(getUrl(), e);
                 }
             }
         }
-        throw protocolException(HttpServletResponse.SC_NOT_FOUND);
+        throw protocolException(getUrl(), HttpServletResponse.SC_NOT_FOUND);
     }
 
     @Override
@@ -180,29 +183,22 @@ public class ContactResource extends CommonResource<Contact> {
         boolean saved = false;
         try {
             if (false == exists()) {
-                throw protocolException(HttpServletResponse.SC_CONFLICT);
+                throw protocolException(getUrl(), HttpServletResponse.SC_CONFLICT);
             } else if (null == vCardImport || null == vCardImport.getContact()) {
-                throw protocolException(HttpServletResponse.SC_NOT_FOUND);
+                throw protocolException(getUrl(), HttpServletResponse.SC_NOT_FOUND);
             }
             /*
              * store original vCard if possible
              */
-            ContactService contactService = factory.requireService(ContactService.class);
             previousVCardID = object.getVCardId();
             Contact contact = vCardImport.getContact();
             vCardFileHolder = vCardImport.getVCard();
-            if (null != vCardFileHolder) {
-                int contextId = factory.getSession().getContextId();
-                VCardStorageService vCardStorageService = factory.getVCardStorageService(contextId);
-                if (vCardStorageService != null) {
-                    vCardID = vCardStorageService.saveVCard(vCardFileHolder.getStream(), contextId);
-                    LOG.debug("{}: saved original vCard in '{}'.", getUrl(), vCardID);
-                    contact.setVCardId(vCardID);
-                }
-            }
+            vCardID = storeVCard(factory.getSession().getContextId(), vCardFileHolder);
+            contact.setVCardId(vCardID);
             /*
              * update contact, trying again in case of recoverable errors
              */
+            ContactService contactService = factory.requireService(ContactService.class);
             for (int i = 0; i < MAX_RETRIES && false == saved; i++) {
                 try {
                     contactService.updateContact(factory.getSession(), Integer.toString(contact.getParentFolderID()), Integer.toString(contact.getObjectID()), contact, contact.getLastModified());
@@ -219,14 +215,14 @@ public class ContactResource extends CommonResource<Contact> {
              */
             handleAttachments(object, contact);
         } catch (OXException e) {
-            throw protocolException(e);
+            throw protocolException(getUrl(), e);
         } finally {
             Streams.close(vCardFileHolder);
             closeVCardImport();
             if (saved) {
-                deleteVCard(previousVCardID);
+                deleteVCard(factory.getSession().getContextId(), previousVCardID);
             } else if (null != vCardID) {
-                deleteVCard(vCardID);
+                deleteVCard(factory.getSession().getContextId(), vCardID);
             }
         }
     }
@@ -237,7 +233,7 @@ public class ContactResource extends CommonResource<Contact> {
         String vCardID = null != object ? object.getVCardId() : null;
         try {
             if (false == exists()) {
-                throw protocolException(HttpServletResponse.SC_NOT_FOUND);
+                throw protocolException(getUrl(), HttpServletResponse.SC_NOT_FOUND);
             }
             /*
              * delete contact, trying again in case of recoverable errors
@@ -257,7 +253,7 @@ public class ContactResource extends CommonResource<Contact> {
             }
         } finally {
             if (null != vCardID && deleted) {
-                deleteVCard(vCardID);
+                deleteVCard(factory.getSession().getContextId(), vCardID);
             }
         }
     }
@@ -269,9 +265,9 @@ public class ContactResource extends CommonResource<Contact> {
         boolean created = false;
         try {
             if (exists()) {
-                throw protocolException(HttpServletResponse.SC_CONFLICT);
+                throw protocolException(getUrl(), HttpServletResponse.SC_CONFLICT);
             } else if (null == vCardImport || null == vCardImport.getContact()) {
-                throw protocolException(HttpServletResponse.SC_NOT_FOUND);
+                throw protocolException(getUrl(), HttpServletResponse.SC_NOT_FOUND);
             }
             /*
              * import vCard as new contact
@@ -294,7 +290,7 @@ public class ContactResource extends CommonResource<Contact> {
              * set initial parent folder to the default contacts folder in case of an iOS client
              */
             contact.setContextId(factory.getSession().getContextId());
-            String parentFolderID = isIOSClient() ? factory.getState().getDefaultFolder().getID() : String.valueOf(this.parentFolderID);
+            String parentFolderID = DAVUserAgent.IOS.equals(getUserAgent()) ? factory.getState().getDefaultFolder().getID() : String.valueOf(this.parentFolderID);
             if (contact.getMarkAsDistribtuionlist()) {
                 /*
                  * insert & delete not supported contact group (next sync cleans up the client)
@@ -306,7 +302,7 @@ public class ContactResource extends CommonResource<Contact> {
                     contactService.createContact(factory.getSession(), parentFolderID, contact);
                     contactService.deleteContact(factory.getSession(), parentFolderID, Integer.toString(contact.getObjectID()), contact.getLastModified());
                 } catch (OXException e) {
-                    throw protocolException(e);
+                    throw protocolException(getUrl(), e);
                 }
                 return;
             }
@@ -314,14 +310,8 @@ public class ContactResource extends CommonResource<Contact> {
              * store original vCard if possible
              */
             vCardFileHolder = vCardImport.getVCard();
-            if (null != vCardFileHolder) {
-                VCardStorageService vCardStorageService = factory.getVCardStorageService(factory.getSession().getContextId());
-                if (null != vCardStorageService) {
-                    vCardID = vCardStorageService.saveVCard(vCardFileHolder.getStream(), factory.getSession().getContextId());
-                    LOG.debug("{}: saved original vCard in '{}'.", getUrl(), vCardID);
-                    contact.setVCardId(vCardID);
-                }
-            }
+            vCardID = storeVCard(factory.getSession().getContextId(), vCardFileHolder);
+            contact.setVCardId(vCardID);
             /*
              * save contact, trying again in case of recoverable errors
              */
@@ -342,12 +332,12 @@ public class ContactResource extends CommonResource<Contact> {
              */
             handleAttachments(null, object);
         } catch (OXException e) {
-            throw protocolException(e);
+            throw protocolException(getUrl(), e);
         } finally {
             Streams.close(vCardFileHolder);
             closeVCardImport();
             if (null != vCardID && false == created) {
-                deleteVCard(vCardID);
+                deleteVCard(factory.getSession().getContextId(), vCardID);
             }
         }
     }
@@ -366,13 +356,17 @@ public class ContactResource extends CommonResource<Contact> {
             /*
              * import vCard and merge with existing contact, ensuring that some important properties don't change
              */
-            String uid = object.getUid();
-            int parentFolderID = object.getParentFolderID();
-            int contextID = object.getContextId();
-            Date lastModified = object.getLastModified();
-            int objectID = object.getObjectID();
-            String vCardID = object.getVCardId();
-            vCardImport = factory.requireService(VCardService.class).importVCard(inputStream, object, parameters);
+            Contact contact = factory.getContactService().getContact(
+                factory.getSession(), String.valueOf(object.getParentFolderID()), String.valueOf(object.getObjectID()));
+            String uid = contact.getUid();
+            int parentFolderID = contact.getParentFolderID();
+            int contextID = contact.getContextId();
+            Date lastModified = contact.getLastModified();
+            int objectID = contact.getObjectID();
+            String vCardID = contact.getVCardId();
+            contact.setProperty("com.openexchange.contact.vcard.photo.uri", PhotoUtils.buildURI(getHostData(), contact));
+            contact.setProperty("com.openexchange.contact.vcard.photo.contentType", contact.getImageContentType());
+            vCardImport = factory.requireService(VCardService.class).importVCard(inputStream, contact, parameters);
             vCardImport.getContact().setUid(uid);
             vCardImport.getContact().setParentFolderID(parentFolderID);;
             vCardImport.getContact().setContextId(contextID);;
@@ -387,35 +381,20 @@ public class ContactResource extends CommonResource<Contact> {
 
     @Override
     protected WebdavProperty internalGetProperty(WebdavProperty property) throws WebdavProtocolException {
-        if (DAVProtocol.CARD_NS.getURI().equals(property.getNamespace()) && "address-data".equals(property.getName())) {
-            if (false == exists()) {
-                return null;
-            }
-            Set<String> propertyNames = new HashSet<String>();
-            List<Element> children = property.getChildren();
-            if (null != children && 0 < children.size()) {
-                propertyNames = new HashSet<String>();
-                for (Element childElement : children) {
-                    if (DAVProtocol.CARD_NS.equals(childElement.getNamespace()) && "prop".equals(childElement.getName())) {
-                        String name = childElement.getAttributeValue("name");
-                        if (null != name) {
-                            propertyNames.add(name);
-                        }
-                    }
-                }
-            }
+        if (exists() && DAVProtocol.CARD_NS.getURI().equals(property.getNamespace()) && "address-data".equals(property.getName())) {
             String value;
+            Set<String> propertyNames = extractRequestedProperties(property);
             if (null != propertyNames && 0 < propertyNames.size()) {
                 try (VCardExport vCardExport = generateVCardResource(propertyNames); InputStream inputStream = vCardExport.getClosingStream()) {
                     value = Streams.stream2string(inputStream, Charsets.UTF_8_NAME);
                 } catch (IOException | OXException e) {
-                    throw protocolException(e);
+                    throw protocolException(getUrl(), e);
                 }
             } else {
                 try (InputStream inputStream = getBody()) {
                     value = Streams.stream2string(inputStream, Charsets.UTF_8_NAME);
                 } catch (IOException e) {
-                    throw protocolException(e);
+                    throw protocolException(getUrl(), e);
                 }
             }
             WebdavProperty result = new WebdavProperty(property.getNamespace(), property.getName());
@@ -436,40 +415,6 @@ public class ContactResource extends CommonResource<Contact> {
         }
     }
 
-    /**
-     * Deletes a vCard silently.
-     *
-     * @param vCardID The identifier of the vCard to delete, or <code>null</code> to do nothing
-     */
-    private void deleteVCard(String vCardID) {
-        if (null != vCardID) {
-            int contextId = factory.getSession().getContextId();
-            VCardStorageService vCardStorage = factory.getVCardStorageService(contextId);
-            if (null != vCardStorage) {
-                try {
-                    vCardStorage.deleteVCard(vCardID, contextId);
-                } catch (OXException e) {
-                    if ("FLS-0017".equals(e.getErrorCode())) {
-                        LOG.debug("vCard file with id {} in context {} no longer found in storage.", vCardID, contextId, e);
-                    } else {
-                        LOG.warn("Error while deleting vCard with id {} in context {} from storage.", vCardID, contextId, e);
-                    }
-                }
-            }
-        }
-    }
-
-    /**
-     * Gets a value indicating whether the request's user agent is assumed to represent an iOS client or not.
-     *
-     * @return <code>true</code> if the request originates in an iOS client, <code>false</code>, otherwise
-     */
-    private boolean isIOSClient() {
-        String userAgent = (String) factory.getSession().getParameter("user-agent");
-        return false == Strings.isEmpty(userAgent) &&
-            Pattern.matches(".*iOS.*dataaccessd.*", userAgent) && false == userAgent.contains("Android");
-    }
-
     /**
      * Tries to handle an exception.
      *
@@ -509,14 +454,14 @@ public class ContactResource extends CommonResource<Contact> {
             LOG.debug("{}: overriding next sync token for client recovery.", this.getUrl());
             this.factory.overrideNextSyncToken();
         } else if (Category.CATEGORY_CONFLICT.equals(e.getCategory())) {
-            throw protocolException(e, HttpServletResponse.SC_CONFLICT);
+            throw protocolException(getUrl(), e, HttpServletResponse.SC_CONFLICT);
         } else if (Category.CATEGORY_SERVICE_DOWN.equals(e.getCategory())) {
             /*
              * throw appropriate protocol exception
              */
-            throw protocolException(e, HttpServletResponse.SC_SERVICE_UNAVAILABLE);
+            throw protocolException(getUrl(), e, HttpServletResponse.SC_SERVICE_UNAVAILABLE);
         } else {
-            throw protocolException(e);
+            throw protocolException(getUrl(), e);
         }
         return false;
     }
@@ -545,7 +490,7 @@ public class ContactResource extends CommonResource<Contact> {
             try {
                 vCardResource = generateVCardResource(null);
             } catch (OXException e) {
-                throw protocolException(e);
+                throw protocolException(getUrl(), e);
             }
         }
         this.vCardExport = reset ? null : vCardResource;
@@ -554,38 +499,165 @@ public class ContactResource extends CommonResource<Contact> {
 
     private VCardExport generateVCardResource(Set<String> propertyNames) throws OXException {
         /*
-         * retrieve an original vCard if available
+         * determine required contact fields for the export
+         */
+        VCardService vCardService = factory.requireService(VCardService.class);
+        VCardParameters parameters = vCardService.createParameters(factory.getSession());
+        ContactField[] contactFields;
+        if (null != propertyNames && 0 < propertyNames.size()) {
+            parameters.setPropertyNames(propertyNames);
+            contactFields = vCardService.getContactFields(propertyNames);
+        } else {
+            contactFields = null;
+        }
+        /*
+         * load required contact data from storage
+         */
+        Contact contact = factory.getContactService().getContact(
+            factory.getSession(), String.valueOf(object.getParentFolderID()), String.valueOf(object.getObjectID()), contactFields);
+        applyAttachments(contact);
+        if (isExportPhotoAsURI() && 0 < contact.getNumberOfImages()) {
+            contact.setProperty("com.openexchange.contact.vcard.photo.uri", PhotoUtils.buildURI(getHostData(), contact));
+            contact.setProperty("com.openexchange.contact.vcard.photo.contentType", contact.getImageContentType());
+        }
+        /*
+         * export contact data & return resulting vCard stream
          */
         InputStream originalVCard = null;
         try {
-            String vCardID = object.getVCardId();
-            if (null != vCardID) {
-                int contextId = factory.getSession().getContextId();
-                VCardStorageService vCardStorage = factory.getVCardStorageService(factory.getSession().getContextId());
-                if (null != vCardStorage) {
-                    try {
-                        originalVCard = vCardStorage.getVCard(vCardID, contextId);
-                    } catch (OXException e) {
-                        LOG.warn("Error retrieving vCard with id {} in context {} from storage.", vCardID, contextId, e);
-                    }
-                }
-            }
-            /*
-             * load required contact data from storage
-             */
-            Contact contact = factory.getContactService().getContact(factory.getSession(), String.valueOf(object.getParentFolderID()), String.valueOf(object.getObjectID()));
-            applyAttachments(contact);
-            /*
-             * export current contact data & return resulting vCard stream
-             */
-            VCardService vCardService = factory.requireService(VCardService.class);
-            VCardParameters parameters = vCardService.createParameters(factory.getSession());
-            if (null != propertyNames && 0 < propertyNames.size()) {
-                parameters.setPropertyNames(propertyNames);
-            }
+            originalVCard = optVCard(factory.getSession().getContextId(), object.getVCardId());
             return vCardService.exportContact(contact, originalVCard, parameters);
         } finally {
             Streams.close(originalVCard);
         }
     }
+
+    /**
+     * Stores a vCard.
+     *
+     * @param contextID The context identifier
+     * @param fileHolder The file holder carrying the vCard data, or <code>null</code> to do nothing
+     * @return The identifier of the stored vCard
+     */
+    private String storeVCard(int contextID, IFileHolder fileHolder) {
+        if (null != fileHolder) {
+            VCardStorageService vCardStorageService = factory.getVCardStorageService(contextID);
+            if (null != vCardStorageService) {
+                try (InputStream inputStream = fileHolder.getStream()) {
+                    String vCardID = vCardStorageService.saveVCard(inputStream, contextID);
+                    LOG.debug("{}: saved vCard in '{}'.", getUrl(), vCardID);
+                    return vCardID;
+                } catch (OXException | IOException e) {
+                    LOG.warn("Error storing vCard in context {}.", contextID, e);
+                }
+            }
+        }
+        return null;
+    }
+
+    /**
+     * Optionally gets the input stream for a stored vCard.
+     *
+     * @param contextID The context identifier
+     * @param vCardID The identifier of the vCard to get, or <code>null</code> to do nothing
+     * @return The vCard, or <code>null</code> if not available
+     */
+    private InputStream optVCard(int contextID, String vCardID) {
+        if (null != vCardID) {
+            VCardStorageService vCardStorage = factory.getVCardStorageService(contextID);
+            if (null != vCardStorage) {
+                try {
+                    return vCardStorage.getVCard(vCardID, contextID);
+                } catch (OXException e) {
+                    LOG.warn("Error retrieving vCard with id {} in context {} from storage.", vCardID, contextID, e);
+                }
+            }
+        }
+        return null;
+    }
+
+    /**
+     * Deletes a vCard silently.
+     *
+     * @param contextID The context identifier
+     * @param vCardID The identifier of the vCard to delete, or <code>null</code> to do nothing
+     * @return <code>true</code> if a vCard file actually has been deleted, <code>false</code>, otherwise
+     */
+    private boolean deleteVCard(int contextID, String vCardID) {
+        if (null != vCardID) {
+            VCardStorageService vCardStorage = factory.getVCardStorageService(contextID);
+            if (null != vCardStorage) {
+                try {
+                    return vCardStorage.deleteVCard(vCardID, contextID);
+                } catch (OXException e) {
+                    if ("FLS-0017".equals(e.getErrorCode())) {
+                        LOG.debug("vCard file with id {} in context {} no longer found in storage.", vCardID, contextID, e);
+                    } else {
+                        LOG.warn("Error while deleting vCard with id {} in context {} from storage.", vCardID, contextID, e);
+                    }
+                }
+            }
+        }
+        return false;
+    }
+
+    /**
+     * Extracts a set of vCard property names as specified in the child nodes of the supplied <code>CARDDAV:address-data</code> property.
+     *
+     * @param addressDataProperty The <code>CARDDAV:address-data</code> property to get the requested vCard properties from
+     * @return The requested vCard properties, or <code>null</code> if no specific properties are requested
+     */
+    private static Set<String> extractRequestedProperties(WebdavProperty addressDataProperty) {
+        List<Element> childElements = addressDataProperty.getChildren();
+        if (null != childElements && 0 < childElements.size()) {
+            Set<String> propertyNames = new HashSet<String>(childElements.size());
+            for (Element childElement : childElements) {
+                if (DAVProtocol.CARD_NS.equals(childElement.getNamespace()) && "prop".equals(childElement.getName())) {
+                    String name = childElement.getAttributeValue("name");
+                    if (null != name) {
+                        propertyNames.add(name);
+                    }
+                }
+            }
+            return propertyNames;
+        }
+        return null;
+    }
+
+    /**
+     * Gets a value indicating whether the value of the <code>PHOTO</code>-property in vCards should be exported as URI or not, based on
+     * the <code>Prefer</code>-header sent by the client.
+     *
+     * @return <code>true</code> if the photo should be exported as URI, <code>false</code>, otherwise
+     */
+    private boolean isExportPhotoAsURI() {
+        /*
+         * evaluate "Prefer" header first
+         */
+        WebDAVRequestContext requestContext = DAVProtocol.getRequestContext();
+        if (null != requestContext) {
+            Enumeration<?> preferHeaders = requestContext.getHeaders("Prefer");
+            if (null != preferHeaders && preferHeaders.hasMoreElements()) {
+                do {
+                    String value = String.valueOf(preferHeaders.nextElement());
+                    if ("photo=uri".equalsIgnoreCase(value)) {
+                        return true;
+                    }
+                    if ("photo=binary".equalsIgnoreCase(value)) {
+                        return false;
+                    }
+                } while (preferHeaders.hasMoreElements());
+            }
+        }
+        /*
+         * default to configuration
+         */
+        try {
+            return "uri".equalsIgnoreCase(factory.getConfigValue("com.openexchange.carddav.preferredPhotoEncoding", "binary"));
+        } catch (OXException e) {
+            LOG.warn("Error getting \"com.openexchange.carddav.preferredPhotoEncoding\", falling back 'binary'.", e);
+        }
+        return false;
+    }
+
 }
